@@ -23,7 +23,6 @@ from poetry.core.packages.project_package import ProjectPackage
 
 from poetry.factory import Factory
 from poetry.installation import Installer
-from poetry.installation.executor import Executor as BaseExecutor
 from poetry.packages import Locker as BaseLocker
 from poetry.repositories import Repository
 from poetry.repositories import RepositoryPool
@@ -32,6 +31,7 @@ from poetry.toml.file import TOMLFile
 from poetry.utils.env import MockEnv
 from poetry.utils.env import NullEnv
 from tests.helpers import MOCK_DEFAULT_GIT_REVISION
+from tests.helpers import TestExecutor
 from tests.helpers import get_dependency
 from tests.helpers import get_package
 
@@ -42,7 +42,6 @@ if TYPE_CHECKING:
     from _pytest.fixtures import FixtureRequest
     from pytest_mock import MockerFixture
 
-    from poetry.installation.operations.operation import Operation
     from poetry.repositories.legacy_repository import LegacyRepository
     from poetry.repositories.pypi_repository import PyPiRepository
     from poetry.utils.env import Env
@@ -50,44 +49,6 @@ if TYPE_CHECKING:
     from tests.types import FixtureDirGetter
 
 
-class Executor(BaseExecutor):
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        super().__init__(*args, **kwargs)
-
-        self._installs: list[Package] = []
-        self._updates: list[Package] = []
-        self._uninstalls: list[Package] = []
-
-    @property
-    def installations(self) -> list[Package]:
-        return self._installs
-
-    @property
-    def updates(self) -> list[Package]:
-        return self._updates
-
-    @property
-    def removals(self) -> list[Package]:
-        return self._uninstalls
-
-    def _do_execute_operation(self, operation: Operation) -> int:
-        ret_val = super()._do_execute_operation(operation)
-
-        if not operation.skipped:
-            getattr(self, f"_{operation.job_type}s").append(operation.package)
-
-        return ret_val
-
-    def _execute_install(self, operation: Operation) -> int:
-        return 0
-
-    def _execute_update(self, operation: Operation) -> int:
-        return 0
-
-    def _execute_uninstall(self, operation: Operation) -> int:
-        return 0
-
-
 class CustomInstalledRepository(InstalledRepository):
     @classmethod
     def load(
@@ -207,7 +168,7 @@ def installer(
         pool,
         config,
         installed=installed,
-        executor=Executor(env, pool, config, NullIO()),
+        executor=TestExecutor(env, pool, config, NullIO()),
     )
 
 
@@ -412,6 +373,8 @@ def _configure_run_install_dev(
 
 
 @pytest.mark.parametrize("lock_version", ("1.1", "2.1"))
+@pytest.mark.parametrize("update", [False, True])
+@pytest.mark.parametrize("requires_synchronization", [False, True])
 @pytest.mark.parametrize(
     ("groups", "installs", "updates", "removals", "with_packages_installed"),
     [
@@ -438,6 +401,8 @@ def test_run_install_with_dependency_groups(
     repo: Repository,
     package: ProjectPackage,
     installed: CustomInstalledRepository,
+    update: bool,
+    requires_synchronization: bool,
     lock_version: str,
 ) -> None:
     _configure_run_install_dev(
@@ -453,10 +418,13 @@ def test_run_install_with_dependency_groups(
     if groups is not None:
         installer.only_groups(groups)
 
-    installer.requires_synchronization(True)
+    installer.update(update)
+    installer.requires_synchronization(requires_synchronization)
     result = installer.run()
     assert result == 0
 
+    if not requires_synchronization:
+        removals = 0
     assert installer.executor.installations_count == installs
     assert installer.executor.updates_count == updates
     assert installer.executor.removals_count == removals
@@ -760,7 +728,7 @@ def test_run_install_with_synchronization(
         *managed_reserved_package_names,
     }
 
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     assert {r.name for r in installer.executor.removals} == expected_removals
 
 
@@ -963,7 +931,7 @@ def test_run_with_optional_and_python_restricted_dependencies(
     # We should only have 2 installs:
     # C,D since python version is not compatible
     # with B's python constraint and A is optional
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     assert installer.executor.installations_count == 2
     assert installer.executor.installations[0].name == "d"
     assert installer.executor.installations[1].name == "c"
@@ -1011,7 +979,7 @@ def test_run_with_optional_and_platform_restricted_dependencies(
     # We should only have 2 installs:
     # C,D since the mocked python version is not compatible
     # with B's python constraint and A is optional
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     assert installer.executor.installations_count == 2
     assert installer.executor.installations[0].name == "d"
     assert installer.executor.installations[1].name == "c"
@@ -1182,7 +1150,7 @@ def test_run_with_conflicting_dependency_extras(
         assert locker.written_data == expected
 
     # Results of installation are consistent with the 'extra' input
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
 
     expected_installations = []
     if extra == "extra-one":
@@ -1275,7 +1243,7 @@ def test_run_with_exclusive_extras_different_sources(
         pool,
         config,
         installed=installed,
-        executor=Executor(
+        executor=TestExecutor(
             MockEnv(),
             pool,
             config,
@@ -1291,7 +1259,7 @@ def test_run_with_exclusive_extras_different_sources(
     if not locked:
         expected = fixture("with-exclusive-extras")
         assert locker.written_data == expected
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     if extra is None:
         assert len(installer.executor.installations) == 0
     else:
@@ -1405,7 +1373,7 @@ def test_run_with_different_dependency_extras(
         pool,
         config,
         installed=installed,
-        executor=Executor(
+        executor=TestExecutor(
             MockEnv(),
             pool,
             config,
@@ -1422,7 +1390,7 @@ def test_run_with_different_dependency_extras(
         assert locker.written_data == expected
 
     # Results of installation are consistent with the 'extra' input
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     if extra is None:
         assert len(installer.executor.installations) == 0
     else:
@@ -1432,6 +1400,7 @@ def test_run_with_different_dependency_extras(
 @pytest.mark.parametrize("is_locked", [False, True])
 @pytest.mark.parametrize("is_installed", [False, True])
 @pytest.mark.parametrize("with_extras", [False, True])
+@pytest.mark.parametrize("do_update", [False, True])
 @pytest.mark.parametrize("do_sync", [False, True])
 def test_run_installs_extras_with_deps_if_requested(
     installer: Installer,
@@ -1442,6 +1411,7 @@ def test_run_installs_extras_with_deps_if_requested(
     is_locked: bool,
     is_installed: bool,
     with_extras: bool,
+    do_update: bool,
     do_sync: bool,
 ) -> None:
     package.extras = {canonicalize_name("foo"): [get_dependency("C")]}
@@ -1475,6 +1445,7 @@ def test_run_installs_extras_with_deps_if_requested(
 
     if with_extras:
         installer.extras(["foo"])
+    installer.update(do_update)
     installer.requires_synchronization(do_sync)
     result = installer.run()
     assert result == 0
@@ -1491,7 +1462,7 @@ def test_run_installs_extras_with_deps_if_requested(
         expected_installations_count = 0 if is_installed else 2
         # We only want to uninstall extras if we do a "poetry install" without extras,
         # not if we do a "poetry update" or "poetry add".
-        expected_removals_count = 2 if is_installed else 0
+        expected_removals_count = 2 if is_installed and do_sync else 0
 
     assert installer.executor.installations_count == expected_installations_count
     assert installer.executor.removals_count == expected_removals_count
@@ -1645,7 +1616,7 @@ def test_run_installs_with_local_poetry_directory_and_skip_directory_flag(
 
     assert locker.written_data == expected
 
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     directory_installs = [
         p.name for p in installer.executor.installations if p.source_type == "directory"
     ]
@@ -1928,7 +1899,7 @@ def test_run_install_duplicate_dependencies_different_constraints(
 
     assert locker.written_data == expected
 
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     installs = installer.executor.installations
     assert installer.executor.installations_count == 3
     assert installs[0] == package_c12
@@ -2302,7 +2273,7 @@ def test_installer_required_extras_should_not_be_removed_when_updating_single_de
         pool,
         config,
         installed=installed,
-        executor=Executor(env, pool, config, NullIO()),
+        executor=TestExecutor(env, pool, config, NullIO()),
     )
     installer.update(True)
     installer.whitelist(["D"])
@@ -2337,7 +2308,7 @@ def test_installer_required_extras_should_not_be_removed_when_updating_single_de
         pool,
         config,
         installed=installed,
-        executor=Executor(env, pool, config, NullIO()),
+        executor=TestExecutor(env, pool, config, NullIO()),
     )
 
     package.add_dependency(
@@ -2359,7 +2330,7 @@ def test_installer_required_extras_should_not_be_removed_when_updating_single_de
     locker.locked(True)
     locker.mock_lock_data(locker.written_data)
 
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     for pkg in installer.executor.installations:
         installed.add_package(pkg)
 
@@ -2371,7 +2342,7 @@ def test_installer_required_extras_should_not_be_removed_when_updating_single_de
         pool,
         config,
         installed=installed,
-        executor=Executor(env, pool, config, NullIO()),
+        executor=TestExecutor(env, pool, config, NullIO()),
     )
     installer.update(True)
     installer.whitelist(["pytest"])
@@ -2403,7 +2374,7 @@ def test_installer_required_extras_should_be_installed(
         pool,
         config,
         installed=installed,
-        executor=Executor(env, pool, config, NullIO()),
+        executor=TestExecutor(env, pool, config, NullIO()),
     )
     package.add_dependency(
         Factory.create_dependency(
@@ -2430,7 +2401,7 @@ def test_installer_required_extras_should_be_installed(
         pool,
         config,
         installed=installed,
-        executor=Executor(env, pool, config, NullIO()),
+        executor=TestExecutor(env, pool, config, NullIO()),
     )
     installer.update(True)
     result = installer.run()
@@ -2552,7 +2523,7 @@ def test_installer_can_install_dependencies_from_forced_source(
         pool,
         config,
         installed=installed,
-        executor=Executor(env, pool, config, NullIO()),
+        executor=TestExecutor(env, pool, config, NullIO()),
     )
     installer.update(True)
     result = installer.run()
@@ -2614,7 +2585,7 @@ def test_run_installs_with_same_version_url_files(
         pool,
         config,
         installed=installed,
-        executor=Executor(
+        executor=TestExecutor(
             MockEnv(platform=env_platform),
             pool,
             config,
@@ -2626,7 +2597,7 @@ def test_run_installs_with_same_version_url_files(
 
     expected = fixture("with-same-version-url-dependencies")
     assert locker.written_data == expected
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     assert installer.executor.installations_count == 2
     demo_package = next(p for p in installer.executor.installations if p.name == "demo")
     assert demo_package.source_url == urls[env_platform]
@@ -2776,7 +2747,7 @@ def test_installer_should_use_the_locked_version_of_git_dependencies(
     result = installer.run()
     assert result == 0
 
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     demo_installation = next(
         package
         for package in installer.executor.installations
@@ -2824,7 +2795,7 @@ def test_installer_should_use_the_locked_version_of_git_dependencies_with_extras
     result = installer.run()
     assert result == 0
 
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     assert len(installer.executor.installations) == 3
     demo_installation = next(
         package
@@ -2869,7 +2840,7 @@ def test_installer_should_use_the_locked_version_of_git_dependencies_without_ref
     result = installer.run()
     assert result == 0
 
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     assert len(installer.executor.installations) == 2
     demo_installation = next(
         package
@@ -2966,7 +2937,7 @@ def test_installer_distinguishes_locked_packages_with_local_version_by_source(
         pool,
         config,
         installed=installed,
-        executor=Executor(
+        executor=TestExecutor(
             MockEnv(platform=env_platform),
             pool,
             config,
@@ -2984,7 +2955,7 @@ def test_installer_distinguishes_locked_packages_with_local_version_by_source(
     )
     source_reference = None if env_platform == "darwin" else "pytorch"
 
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     assert len(installer.executor.installations) == 1
     assert installer.executor.installations[0] == Package(
         "torch",
@@ -3073,7 +3044,7 @@ def test_installer_distinguishes_locked_packages_with_same_version_by_source(
         pool,
         config,
         installed=installed,
-        executor=Executor(
+        executor=TestExecutor(
             MockEnv(platform_machine=env_platform_machine),
             pool,
             config,
@@ -3094,7 +3065,7 @@ def test_installer_distinguishes_locked_packages_with_same_version_by_source(
         source_url = None
         source_reference = None
 
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     assert len(installer.executor.installations) == 1
     assert installer.executor.installations[0] == Package(
         "kivy",
@@ -3204,7 +3175,7 @@ def test_explicit_source_dependency_with_direct_origin_dependency(
         pool,
         config,
         installed=installed,
-        executor=Executor(
+        executor=TestExecutor(
             MockEnv(platform=env_platform),
             pool,
             config,
@@ -3215,7 +3186,7 @@ def test_explicit_source_dependency_with_direct_origin_dependency(
     result = installer.run()
 
     assert result == 0
-    assert isinstance(installer.executor, Executor)
+    assert isinstance(installer.executor, TestExecutor)
     if env_platform == "linux":
         assert set(installer.executor.installations) == {
             Package("pendulum", "1.4.4"),
